@@ -4,10 +4,10 @@ import logging
 import warnings
 
 from django.db import models
-from django.db.models import signals, Count
+from django.db.models import signals, Sum, F
 
 from sentry.conf import settings
-from sentry.utils import construct_checksum, get_db_engine
+from sentry.utils import construct_checksum, get_db_engine, should_mail
 from sentry.utils.charts import has_charts
 from sentry.utils.compat.db import connections
 
@@ -34,9 +34,36 @@ class ScoreClause(object):
         else:
             # XXX: if we cant do it atomicly let's do it the best we can
             sql = self.group.get_score()
-        
+
         return (sql, [])
 
+
+def count_limit(count):
+    # TODO: could we do something like num_to_store = max(math.sqrt(100*count)+59, 200) ?
+    # ~ 150 * ((log(n) - 1.5) ^ 2 - 0.25)
+    if count <= 50: # 200
+        return 1
+    if count <= 1000: # 400
+        return 2
+    if count <= 10000: # 900
+        return 10
+    if count <= 100000: # 1800
+        return 50
+    if count <= 1000000: # 3000
+        return 300
+    if count <= 10000000: # 4500 
+        return 2000
+    return 10000
+
+def time_limit(silence): # ~ 3600 per hour  
+    if silence >= 3600:
+        return 1
+    if silence >= 360:
+        return 10
+    if silence >= 60:
+        return 60
+    return 10000
+
 class SentryManager(models.Manager):
     use_for_related_fields = True
 
@@ -46,9 +73,10 @@ class SentryManager(models.Manager):
             qs = qs.using(settings.DATABASE_USING)
         return qs
 
+
     def from_kwargs(self, **kwargs):
         from sentry.models import Message, GroupedMessage, FilterValue
-        
+
         URL_MAX_LENGTH = Message._meta.get_field_by_name('url')[0].max_length
         now = kwargs.pop('timestamp', None) or datetime.datetime.now()
 
@@ -59,7 +87,7 @@ class SentryManager(models.Manager):
         site = kwargs.pop('site', None)
         data = kwargs.pop('data', {}) or {}
         message_id = kwargs.pop('message_id', None)
-        
+
         if url:
             data['url'] = url
             url = url[:URL_MAX_LENGTH]
@@ -97,11 +125,13 @@ class SentryManager(models.Manager):
                 # HACK: maintain appeared state
                 if group.status == 1:
                     mail = True
+                silence_timedelta = now - group.last_seen
+                silence = silence_timedelta.days * 86400 + silence_timedelta.seconds
                 group.status = 0
                 group.last_seen = now
                 group.times_seen += 1
                 GroupedMessage.objects.filter(pk=group.pk).update(
-                    times_seen=models.F('times_seen') + 1,
+                    times_seen=F('times_seen') + 1,
                     status=0,
                     last_seen=now,
                     score=ScoreClause(group),
@@ -111,10 +141,10 @@ class SentryManager(models.Manager):
                 GroupedMessage.objects.filter(pk=group.pk).update(
                     score=ScoreClause(group),
                 )
+                silence = 0
                 mail = True
 
-                
-            instance = Message.objects.create(
+            instance = Message(
                 message_id=message_id,
                 view=view,
                 logger=logger_name,
@@ -127,12 +157,42 @@ class SentryManager(models.Manager):
                 datetime=now,
                 **kwargs
             )
-            if server_name:
-                FilterValue.objects.get_or_create(key='server_name', value=server_name)
-            if site:
-                FilterValue.objects.get_or_create(key='site', value=site)
-            if logger_name:
-                FilterValue.objects.get_or_create(key='logger', value=logger_name)
+
+            if not settings.SAMPLE_DATA or group.times_seen % min(count_limit(group.times_seen), time_limit(silence)) == 0:
+                instance.save()
+
+            # rounded down to the nearest interval
+            if settings.MINUTE_NORMALIZATION:
+                minutes = (now.minute - (now.minute % settings.MINUTE_NORMALIZATION))
+            else:
+                minutes = now.minute
+            normalized_datetime = now.replace(second=0, microsecond=0, minute=minutes)
+
+            affected = group.messagecountbyminute_set.filter(date=normalized_datetime).update(times_seen=F('times_seen') + 1)
+            if not affected:
+                group.messagecountbyminute_set.create(
+                    date=normalized_datetime,
+                    times_seen=1,
+                )
+
+            for key, value in (
+                    ('server_name', server_name),
+                    ('site', site),
+                    ('logger', logger_name)
+                ):
+                if not value:
+                    continue
+
+                FilterValue.objects.get_or_create(key=key, value=value)
+
+                affected = group.messagefiltervalue_set.filter(key=key, value=value).update(times_seen=F('times_seen') + 1)
+                if not affected:
+                    group.messagefiltervalue_set.create(
+                        key=key,
+                        value=value,
+                        times_seen=1,
+                    )
+
         except Exception, exc:
             # TODO: should we mail admins when there are failures?
             try:
@@ -140,8 +200,9 @@ class SentryManager(models.Manager):
             except Exception, exc:
                 warnings.warn(u'Unable to process log entry: %s' % (exc,))
         else:
-            if mail:
+            if mail and should_mail(group):
                 group.mail_admins()
+
             return instance
 
 class GroupedMessageManager(SentryManager):
@@ -161,18 +222,18 @@ class GroupedMessageManager(SentryManager):
 
         engine = get_db_engine(db)
         if engine.startswith('oracle'):
-            method = conn.ops.date_trunc_sql('hh24', 'datetime')
+            method = conn.ops.date_trunc_sql('hh24', 'date')
         else:
-            method = conn.ops.date_trunc_sql('hour', 'datetime')
+            method = conn.ops.date_trunc_sql('hour', 'date')
 
         hours = max_days*24
         today = datetime.datetime.now().replace(microsecond=0, second=0, minute=0)
         min_date = today - datetime.timedelta(hours=hours)
 
-        chart_qs = list(group.message_set.all()\
-                          .filter(datetime__gte=min_date)\
+        chart_qs = list(group.messagecountbyminute_set.all()\
+                          .filter(date__gte=min_date)\
                           .extra(select={'grouper': method}).values('grouper')\
-                          .annotate(num=Count('id')).values_list('grouper', 'num')\
+                          .annotate(num=Sum('times_seen')).values_list('grouper', 'num')\
                           .order_by('grouper'))
 
         if not chart_qs:
